class BasicRelationalOperator:
    @classmethod
    def __get_operator_name__(cls):
        return cls.__name__


class BasicLogicalOperator:
    @classmethod
    def __get_operator_name__(cls):
        return cls.__name__


class BasicFunction:
    @classmethod
    def __get_operator_name__(cls):
        return cls.__name__


class BasicUI:
    @classmethod
    def __get_operator_name__(cls):
        return cls.__name__


# 关系运算符
class Equal(BasicRelationalOperator):
    ...


class NotEqual(BasicRelationalOperator):
    ...


class Exists(BasicRelationalOperator):
    ...


class NotExists(BasicRelationalOperator):
    ...


class GreaterThan(BasicRelationalOperator):
    ...


class LessThan(BasicRelationalOperator):
    ...


class GreaterThanOrEqual(BasicRelationalOperator):
    ...


class LessThanOrEqual(BasicRelationalOperator):
    ...



from dp.launching.typing.basic import BaseModel
from typing import Any, Optional, Callable
from dataclasses import dataclass, field


@dataclass
class TableBasicProps:
    df: Any
    columns: list
    row_key: Optional[list] = field(default=None)
    hidden_columns: Optional[list] = field(default=None)
    fixed_left_columns: Optional[list] = field(default=None)
    fixed_right_columns: Optional[list] = field(default=None)
    custom_columns_width: Optional[dict] = field(default=None)
    default_column_width: int = field(default=None)
    tags_columns: Optional[list] = field(default=None)
    sorter_columns: Optional[list] = field(default=None)
    searchable_columns: Optional[list] = field(default=None)
    iframes_mapper: Optional[Callable] = field(default=None)
    actions: Optional[list] = field(default=None)
    actions_mapper: Optional[Callable] = field(default=None)
    iframe_height: int = field(default=300)
    default_expand_all_rows: Optional[bool] = field(default=False)
    batch_actions: Optional[list] = field(default=None)
    linkable_columns: Optional[list] = field(default=None)
    expand_column: Optional[str] = field(default=None)
    expand_json: Optional[bool] = field(default=False)
    action_width: Optional[int] = field(default=None)
    compact_layout: bool = field(default=False)
    show_pager: bool = field(default=True)
    color_backgroud: str = field(default="#f0f0f0")
    rows_per_page: int = field(default=20)
    min_height: int = field(default=200)
    enable_dynamic_pager: bool = field(default=False)
    dynamic_pager_total: int = field(default=0)
    dynamic_pager_page: int = field(default=0)
    unsafe_html_columns: Optional[list] = field(default=None)
    key: Optional[str] = field(default=None)


@dataclass
class DynamicTableProps(TableBasicProps):
    key: Optional[str] = field(default=None)
    df_loader: Callable = field(default=None)
    total: int = field(default=None)
    page_size: int = field(default=10)
    state: Optional[Any] = field(default=None)


class DynamicTable(BaseModel):
    Props = DynamicTableProps
    props: DynamicTableProps
    format = "dynamic_table"
    scope = "preview"


DynamicTable.Props = DynamicTableProps


class Table(BaseModel):
    Props = TableBasicProps
    props: TableBasicProps
    format = "table"
    scope = "preview"



from typing import Any, Type, get_args, Optional
import dict_deep
from dp.launching.typing.basic import Union, Dict, Optional, BaseModel, get_internal_key
from dp.launching.typing.addon.sysmbol import (
    BasicRelationalOperator,
    BasicLogicalOperator,
    BasicFunction,
)


class UI:
    ...


def get_wrapper(type: str) -> Type[BaseModel]:
    class _UI(UI):
        def __init__(self, *args: Any, **kwargs: Any) -> None:
            self.args = args
            self.kwargs = kwargs

        def __get_field_schema__(self) -> Any:
            args = self.args
            others = getOthers(args)
            operator = getBasicRelationalOperator(args)
            left = getCurrent(args)
            ref = getRef(args)
            internal_key = get_internal_key()
            refChain = getRefChain(ref)
            return {
                "scope": "internal",
                "statements": "ui",
                "value": type,
                # "group": self.__get_group_info__(),
                "params": {
                    "ref": refChain,
                    "left": (left or ref).__get_id__() if left or ref else None,
                    "operator": operator.__get_operator_name__() if operator else None,
                    "others": others,
                },
            }

        @classmethod
        def __get_group_info__(cls) -> dict:
            return (
                cls.__group_info__
                if hasattr(cls, "__group_info__")
                else {"name": "", "desc": ""}
            )

        def __call__(self, cls) -> Any:
            args = self.args
            others = getOthers(args)
            operator = getBasicRelationalOperator(args)
            left = getCurrent(args)
            ref = getRef(args)
            internal_key = get_internal_key()
            refChain = getRefChain(ref)

            class _internal_ui_property:
                __group_info__ = None

                @classmethod
                def __set_group_info__(cls, name: str, desc: str = "") -> None:
                    cls.__group_info__ = {"name": name, "desc": desc}

                @classmethod
                def __get_group_info__(cls) -> dict:
                    return (
                        cls.__group_info__
                        if hasattr(cls, "__group_info__")
                        else {"name": "", "desc": ""}
                    )

                @classmethod
                def __modify_schema__(cls, field_schema: Dict[str, Any]) -> None:
                    field_schema.update(
                        scope="internal",
                        statements="ui",
                        value=type,
                        group=cls.__get_group_info__(),
                        params={
                            "ref": refChain,
                            "left": (left or ref).__get_id__()
                            if (left or ref)
                            else None,
                            "operator": operator.__get_operator_name__()
                            if operator
                            else None,
                            "others": others,
                        },
                    )

                @classmethod
                def __get_validators__(cls) -> Any:
                    yield cls.validate

                @classmethod
                def validate(cls, value: Any) -> "_internal_ui_property":
                    return _internal_ui_property

            class _stand(_Basic_internal_ref_cls, cls):
                __annotations__ = {internal_key: Optional[_internal_ui_property]}

            _stand.__qualname__ = f"{cls.__qualname__}"
            _stand.__name__ = f"{cls.__name__}"
            _stand.current = cls
            cls._stand = _stand
            _stand.__get_fields__()

            return _stand

    return _UI


class _Basic_internal_ref_cls:
    @classmethod
    def get_current(cls):
        return cls.current


# def getUI(args: tuple) -> UI:
#     for arg in args:
#         if isinstance(arg, type) and issubclass(arg, UI):
#             return arg


def getBasicRelationalOperator(args: tuple) -> BasicRelationalOperator:
    for arg in args:
        if isinstance(arg, type) and issubclass(arg, BasicRelationalOperator):
            return arg


def getBasicLogicalOperator(args: tuple) -> BasicLogicalOperator:
    for arg in args:
        if isinstance(arg, type) and issubclass(arg, BasicLogicalOperator):
            return arg


def is_internal_cls(target: Any) -> bool:
    if not isinstance(target, type):
        return False

    if (
        not issubclass(target, BasicLogicalOperator)
        and not issubclass(target, BasicRelationalOperator)
        and not issubclass(target, BasicFunction)
        and not issubclass(target, BaseModel)
    ):
        # and not issubclass(target, UI) and
        return False
    return True


def getOthers(args: tuple) -> Optional[tuple]:
    res = []
    for arg in args:
        if not is_internal_cls(arg):
            res.append(arg)
    return tuple(res)


def getCurrent(args: tuple) -> Optional[BaseModel]:
    for arg in args:
        if isinstance(arg, type) and issubclass(arg, _Basic_internal_ref_cls):
            return arg.get_current()


def getRef(args: BaseModel) -> Optional[BaseModel]:
    for arg in args:
        if isinstance(arg, type) and issubclass(arg, BaseModel):
            return arg


def getRefChain(ref):
    if not ref or not hasattr(ref, "origin_schema"):
        return None
    if not isinstance(ref, type) or not issubclass(ref, _Basic_internal_ref_cls):
        return None
    sch = ref.origin_schema()
    parent = dict_deep.deep_get(sch, f"properties.{get_internal_key()}.params")
    return {
        "schema": sch,
        "parent": parent,
    }


Visible = get_wrapper("Visible")

Hidden = get_wrapper("Hidden")


class Group:
    def __init__(self, name: str, desc: str = "") -> None:
        self.name = name
        self.desc = desc

    # @classmethod
    # def __set_group_info__(cls, name: str, desc: str = "") -> None:
    #     cls.__group_info__ = {"name": name, "desc": desc}

    def __add_group__(self, cls) -> dict:
        _internal_ui_property = cls._stand.__annotations__[get_internal_key()]
        if get_args(_internal_ui_property):
            for u in get_args(_internal_ui_property):
                if hasattr(u, "__set_group_info__"):
                    u.__set_group_info__(self.name, self.desc)
        elif hasattr(_internal_ui_property, "__set_group_info__"):
            _internal_ui_property.__set_group_info__(self.name, self.desc)
        return cls._stand

    def __call__(self, cls) -> Any:
        if hasattr(cls, "_stand"):
            return self.__add_group__(cls)
        else:
            cls = Visible(None, None, None)(cls)
            return self.__add_group__(cls)



"""
# This is title
markdown content here
"""

from dp.launching.typing import BaseModel, Field

from dp.launching.typing import InputFilePath, OutputDirectory
from dp.launching.typing import List, Union, Optional, Set, Enum
from dp.launching.typing import String, Int, Boolean, Float, DateTime, Date

from dp.launching.typing.dflow import (
    DflowArgoAPIServer,
    DflowK8sAPIServer,
    DflowAccessToken,
    DflowStorageEndpoint,
    DflowStorageRepository,
)

from dp.launching.typing.bohrium import (
    BohriumJobType,
    BohriumMachineType,
    BohriumPlatform,
    BohriumProjectId,
    BohriumUsername,
    BohriumPassword,
    BohriumImage,
)

from dp.launching.cli import to_runner, default_minimal_exception_handler


class IOOptions(BaseModel):
    input_protein: InputFilePath = Field(
        ..., ftypes=["pdbqt", "pdf"], description="Protein file path"
    )
    input_ligands: List[InputFilePath] = Field(
        ..., ftypes=["pdbqt", "sdf"], description="Ligands for Job"
    )
    output_dir: OutputDirectory = Field(
        default="./output"
    )  # defaut will be override after online


class AlgorithmMode(str, Enum):
    """_summary_"""

    quick = "quick"
    default = "default"


class SetOptional(String, Enum):
    option_i_1 = "option_i_1"
    option_i_2 = "option_i_2"


class AlgorithmOptions(BaseModel):
    mode: AlgorithmMode = Field(default=AlgorithmMode.quick, description="balance mode")
    option_a: Int
    option_b: Float
    option_c: String
    option_d: Boolean
    option_e: Optional[String]
    option_f: DateTime
    option_g: Date
    option_h: SetOptional = Field(description="Only select a single item from a set.")
    option_i: Set[SetOptional] = Field(description="Allows multiple items from a set.")


class DflowOptions(BaseModel):
    """
    For dflow Jobs, All variable will be injected
    """

    dflow_argo_api_server: DflowArgoAPIServer
    dflow_k8s_api_server: DflowK8sAPIServer
    dflow_access_token: DflowAccessToken
    dflow_storage_endpoint: DflowStorageEndpoint
    dflow_storage_repository: DflowStorageRepository


class GlobalOptions(IOOptions, AlgorithmOptions, DflowOptions, BaseModel):
    ...


def example_runner(opts: GlobalOptions) -> int:
    status = 0
    from pprint import pprint

    print("Opts:")
    pprint(opts.dict())
    return status


__all__ = ["GlobalOptions"]


def to_parser():
    return to_runner(
        GlobalOptions,
        example_runner,
        version="0.1.0",
        exception_handler=default_minimal_exception_handler,
    )


if __name__ == "__main__":
    import sys

    to_parser()(sys.argv[1:])



from typing import Any, Dict

__all__ = ["DatasetsSelect", "ModelsSelect"]


class DatasetsSelect(object):
    datahub_type = "datasets"

    def get_value(self):
        if hasattr(self, "value"):
            return self.value
        return None

    @classmethod
    def __modify_schema__(cls, field_schema: Dict[str, Any]) -> None:
        field_schema.update(format="datasets_select", scope="io")

    @classmethod
    def __get_validators__(cls) -> Any:
        yield cls.validate

    @classmethod
    def validate(cls, value: Any) -> "DatasetsSelect":
        if isinstance(value, DatasetsSelect):
            return value
        elif isinstance(value, str):
            ds = DatasetsSelect()
            ds.value = value
            return ds
        elif isinstance(value, list):
            ds = DatasetsSelect()
            ds.value = value
            return ds
        else:
            raise Exception(f"DatasetsSelect Wrong type: {type(value)}")


class ModelsSelect(object):
    type = "models"

    def get_value(self):
        if hasattr(self, "value"):
            return self.value
        return None

    @classmethod
    def __modify_schema__(cls, field_schema: Dict[str, Any]) -> None:
        field_schema.update(format="models_select", scope="io")

    @classmethod
    def __get_validators__(cls) -> Any:
        yield cls.validate

    @classmethod
    def validate(cls, value: Any) -> "ModelsSelect":
        if isinstance(value, ModelsSelect):
            return value
        elif isinstance(value, str):
            ds = ModelsSelect()
            ds.value = value
            return ds
        elif isinstance(value, list):
            ds = ModelsSelect()
            ds.value = value
            return ds
        else:
            raise Exception(f"ModelsSelect Wrong type: {type(value)}")



import base64
from typing import Any, Dict

__all__ = [
    "BohriumJobType",
    "BohriumMachineType",
    "BohriumProjectId",
    "BohriumPlatform",
    "BohriumUsername",
    "BohriumPassword",
    "BohriumTicket",
    "BohriumImage",
    "BohriumInstanceType",
]


class BohriumUsername(str):
    def get_value(self):
        return str(self)

    @classmethod
    def __modify_schema__(cls, field_schema: Dict[str, Any]) -> None:
        field_schema.update(format="bohrium_username", scope="executor")

    @classmethod
    def __get_validators__(cls) -> Any:
        yield cls.validate

    @classmethod
    def validate(cls, value: Any) -> "BohriumUsername":
        if isinstance(value, BohriumUsername):
            return value
        elif isinstance(value, str):
            return BohriumUsername(value)
        elif isinstance(value, (bytes, bytearray, memoryview)):
            return BohriumUsername(base64.b64encode(value).decode())
        else:
            raise Exception(f"BohriumUsername Wrong type: {type(value)}")


class BohriumPassword(str):
    def get_value(self):
        return str(self)

    @classmethod
    def __modify_schema__(cls, field_schema: Dict[str, Any]) -> None:
        field_schema.update(format="bohrium_password", scope="executor")

    @classmethod
    def __get_validators__(cls) -> Any:
        yield cls.validate

    @classmethod
    def validate(cls, value: Any) -> "BohriumPassword":
        if isinstance(value, BohriumPassword):
            return value
        elif isinstance(value, str):
            return BohriumPassword(value)
        elif isinstance(value, (bytes, bytearray, memoryview)):
            return BohriumPassword(base64.b64encode(value).decode())
        else:
            raise Exception(f"BohriumPassword Wrong type: {type(value)}")


class BohriumTicket(str):
    def get_value(self):
        return str(self)

    @classmethod
    def __modify_schema__(cls, field_schema: Dict[str, Any]) -> None:
        field_schema.update(format="bohrium_ticket", scope="executor")

    @classmethod
    def __get_validators__(cls) -> Any:
        yield cls.validate

    @classmethod
    def validate(cls, value: Any) -> "BohriumTicket":
        if isinstance(value, BohriumTicket):
            return value
        elif isinstance(value, str):
            return BohriumTicket(value)
        elif isinstance(value, (bytes, bytearray, memoryview)):
            return BohriumTicket(base64.b64encode(value).decode())
        else:
            raise Exception(f"BohriumTicket Wrong type: {type(value)}")


class BohriumProjectId(str):
    def get_value(self):
        return str(self)

    @classmethod
    def __modify_schema__(cls, field_schema: Dict[str, Any]) -> None:
        field_schema.update(format="bohrium_project_id", scope="executor")

    @classmethod
    def __get_validators__(cls) -> Any:
        yield cls.validate

    @classmethod
    def validate(cls, value: Any) -> "BohriumProjectId":
        if isinstance(value, BohriumProjectId):
            return value
        elif isinstance(value, str):
            return BohriumProjectId(value)
        elif isinstance(value, (bytes, bytearray, memoryview)):
            return BohriumProjectId(base64.b64encode(value).decode())
        else:
            raise Exception(f"BohriumProjectId Wrong type: {type(value)}")


class BohriumImage(str):
    def get_value(self):
        return str(self)

    @classmethod
    def __modify_schema__(cls, field_schema: Dict[str, Any]) -> None:
        field_schema.update(format="bohrium_image", scope="executor")

    @classmethod
    def __get_validators__(cls) -> Any:
        yield cls.validate

    @classmethod
    def validate(cls, value: Any) -> "BohriumImage":
        if isinstance(value, BohriumImage):
            return value
        elif isinstance(value, str):
            return BohriumImage(value)
        elif isinstance(value, (bytes, bytearray, memoryview)):
            return BohriumImage(base64.b64encode(value).decode())
        else:
            raise Exception(f"BohriumImage Wrong type: {type(value)}")


class BohriumJobType(str):
    INDICATE: "BohriumJobType"
    CONTAINER: "BohriumJobType"

    def __get_default_options__(self):
        return (
            self.CONTAINER,
            self.INDICATE,
        )

    def get_value(self):
        return str(self)

    @classmethod
    def __modify_schema__(cls, field_schema: Dict[str, Any]) -> None:
        field_schema.update(format="bohrium_job_type", scope="executor")

    @classmethod
    def __get_validators__(cls) -> Any:
        yield cls.validate

    @classmethod
    def validate(cls, value: Any) -> "BohriumJobType":
        if isinstance(value, BohriumJobType):
            return value
        elif isinstance(value, str):
            return BohriumJobType(value)
        elif isinstance(value, (bytes, bytearray, memoryview)):
            return BohriumJobType(base64.b64encode(value).decode())
        else:
            raise Exception(f"BohriumJobType Wrong type: {type(value)}")


BohriumJobType.INDICATE = BohriumJobType("indicate")
BohriumJobType.CONTAINER = BohriumJobType("container")


class BohriumMachineType(str):
    C2_M8_CPU: "BohriumMachineType"
    C4_M16_CPU: "BohriumMachineType"
    C8_M32_CPU: "BohriumMachineType"
    C8_M31_1__NVIDIA_T4: "BohriumMachineType"
    C8_M32_1__NVIDIA_V100: "BohriumMachineType"
    C8_M60_1__NVIDIA_P100: "BohriumMachineType"

    def __get_default_options__(self):
        return (
            self.C2_M8_CPU,
            self.C4_M16_CPU,
            self.C8_M32_CPU,
            self.C8_M31_1__NVIDIA_T4,
            self.C8_M32_1__NVIDIA_V100,
            self.C8_M60_1__NVIDIA_P100,
        )

    def get_value(self):
        return str(self)

    @classmethod
    def __modify_schema__(cls, field_schema: Dict[str, Any]) -> None:
        field_schema.update(format="bohrium_machine_type", scope="executor")

    @classmethod
    def __get_validators__(cls) -> Any:
        yield cls.validate

    @classmethod
    def validate(cls, value: Any) -> "BohriumMachineType":
        if isinstance(value, BohriumMachineType):
            return value
        elif isinstance(value, str):
            return BohriumMachineType(value)
        elif isinstance(value, (bytes, bytearray, memoryview)):
            return BohriumMachineType(base64.b64encode(value).decode())
        else:
            raise Exception(f"BohriumMachineType Wrong type: {type(value)}")


BohriumMachineType.C2_M8_CPU = BohriumMachineType("c2_m8_cpu")
BohriumMachineType.C4_M16_CPU = BohriumMachineType("c4_m16_cpu")
BohriumMachineType.C8_M32_CPU = BohriumMachineType("c8_m32_cpu")
BohriumMachineType.C8_M31_1__NVIDIA_T4 = BohriumMachineType("c8_m31_1 * NVIDIA T4")
BohriumMachineType.C8_M32_1__NVIDIA_V100 = BohriumMachineType("c8_m32_1 * NVIDIA V100")
BohriumMachineType.C8_M60_1__NVIDIA_P100 = BohriumMachineType("c8_m60_1 * NVIDIA P100")


class BohriumPlatform(str):
    ALI: "BohriumPlatform"
    PARATERA: "BohriumPlatform"

    def __get_default_options__(self):
        return (
            self.ALI,
            self.PARATERA,
        )

    def get_value(self):
        return str(self)

    @classmethod
    def __modify_schema__(cls, field_schema: Dict[str, Any]) -> None:
        field_schema.update(format="bohrium_platform", scope="executor")

    @classmethod
    def __get_validators__(cls) -> Any:
        yield cls.validate

    @classmethod
    def validate(cls, value: Any) -> "BohriumPlatform":
        if isinstance(value, BohriumPlatform):
            return value
        elif isinstance(value, str):
            return BohriumPlatform(value)
        elif isinstance(value, (bytes, bytearray, memoryview)):
            return BohriumPlatform(base64.b64encode(value).decode())
        else:
            raise Exception(f"BohriumPlatform Wrong type: {type(value)}")


BohriumPlatform.ALI = BohriumPlatform("ali")
BohriumPlatform.PARATERA = BohriumPlatform("paratera")


class BohriumInstanceType(str):
    ON_DEMAND: "BohriumInstanceType"
    SPOT: "BohriumInstanceType"

    def __get_default_options__(self):
        return (
            self.SPOT,
            self.ON_DEMAND,
        )

    def get_value(self):
        return str(self)

    @classmethod
    def __modify_schema__(cls, field_schema: Dict[str, Any]) -> None:
        field_schema.update(format="bohrium_instance_type", scope="executor")

    @classmethod
    def __get_validators__(cls) -> Any:
        yield cls.validate

    @classmethod
    def validate(cls, value: Any) -> "BohriumInstanceType":
        if isinstance(value, BohriumInstanceType):
            return value
        elif isinstance(value, str):
            return BohriumInstanceType(value)
        else:
            raise Exception(f"BohriumInstanceType Wrong type: {type(value)}")


BohriumInstanceType.ON_DEMAND = BohriumInstanceType("On Demand")
BohriumInstanceType.SPOT = BohriumInstanceType("SPOT")



import os
import base64
import os.path
import json
from typing import Any, Dict

__all__ = [
    "InputFilePath",
    "OutputDirectory",
    "DataSet",
    "Model",
    "OutputCommandDirectory",
    "InputMoleculeContent",
]


class InputFilePath(str):
    def get_full_path(self) -> str:
        path = self.get_path()
        return os.path.abspath(path)

    def get_path(self) -> str:
        if hasattr(self, "path"):
            return self.path
        return ""

    def get_name(self) -> str:
        if hasattr(self, "name"):
            return self.name
        return ""

    def get_value(self) -> bytes:
        if hasattr(self, "value"):
            return self.value
        return ""

    def __str__(self):
        if hasattr(self, "path"):
            return self.get_path()
        return ""

    @classmethod
    def __modify_schema__(cls, field_schema: Dict[str, Any]) -> None:
        field_schema.update(format="input_file_path", scope="io")

    @classmethod
    def __get_validators__(cls) -> Any:  # type: ignore
        yield cls.validate

    @classmethod
    def validate(cls, value: Any) -> "InputFilePath":
        if isinstance(value, str):
            f = InputFilePath(value)
            f.path = value
            f.value = value
            f.name = os.path.basename(value)
            return f
        elif isinstance(value, dict):
            if "content" in value:
                f = InputFilePath(value["content"])
                f.value = value["content"]
                if "name" in value:
                    f.name = value["name"]
                return f
            return None
        else:
            raise Exception(f"InputFilePath Wrong type: {type(value)}")


class OutputDirectory(str):
    def get_full_path(self) -> str:
        path = self.get_path()
        return os.path.abspath(path)

    def get_path(self) -> str:
        return self.get_value()

    def get_value(self):
        return str(self)

    @classmethod
    def __modify_schema__(cls, field_schema: Dict[str, Any]) -> None:
        field_schema.update(format="output_directory", scope="io")

    @classmethod
    def __get_validators__(cls) -> Any:
        yield cls.validate

    @classmethod
    def validate(cls, value: Any) -> "OutputDirectory":
        if isinstance(value, OutputDirectory):
            return value
        elif isinstance(value, str):
            return OutputDirectory(value)
        elif isinstance(value, (bytes, bytearray, memoryview)):
            return OutputDirectory(base64.b64encode(value).decode())
        else:
            raise Exception(f"OutputDirectory Wrong type: {type(value)}")


class OutputCommandDirectory(str):
    def get_value(self):
        return str(self)

    @classmethod
    def __modify_schema__(cls, field_schema: Dict[str, Any]) -> None:
        field_schema.update(format="output_command_directory", scope="io")

    @classmethod
    def __get_validators__(cls) -> Any:
        yield cls.validate

    @classmethod
    def validate(cls, value: Any) -> "OutputCommandDirectory":
        if isinstance(value, OutputCommandDirectory):
            return value
        elif isinstance(value, str):
            return OutputCommandDirectory(value)
        elif isinstance(value, (bytes, bytearray, memoryview)):
            return OutputCommandDirectory(base64.b64encode(value).decode())
        else:
            raise Exception(f"OutputCommandDirectory Wrong type: {type(value)}")


class OutputCommandScript(str):
    def get_value(self):
        return str(self)

    @classmethod
    def __modify_schema__(cls, field_schema: Dict[str, Any]) -> None:
        field_schema.update(format="output_command_script", scope="io")

    @classmethod
    def __get_validators__(cls) -> Any:
        yield cls.validate

    @classmethod
    def validate(cls, value: Any) -> "OutputCommandScript":
        if isinstance(value, OutputCommandScript):
            return value
        elif isinstance(value, str):
            return OutputCommandScript(value)
        elif isinstance(value, (bytes, bytearray, memoryview)):
            return OutputCommandScript(base64.b64encode(value).decode())
        else:
            raise Exception(f"OutputCommandScript Wrong type: {type(value)}")


class DataSet(str):
    def get_value(self) -> Dict[str, Any]:
        return json.loads(str(self))

    def get_path(self) -> str:
        return self.get_value()["path"]

    def get_subpath(self) -> str:
        return self.get_value()["subpath"]

    def get_full_path(self) -> str:
        path = self.get_path()
        return os.path.abspath(path)

    def get_urn(self) -> str:
        return self.get_value()["urn"]

    def get_name(self) -> str:
        return self.get_value().get("name")

    @classmethod
    def __modify_schema__(cls, field_schema: Dict[str, Any]) -> None:
        field_schema.update(format="dataset", scope="io")

    @classmethod
    def __get_validators__(cls) -> Any:
        yield cls.validate

    @classmethod
    def validate(cls, value: Any) -> "DataSet":
        if isinstance(value, DataSet):
            return value
        elif isinstance(value, str):
            if os.path.exists(value):
                return DataSet(json.dumps({"path": value, "urn": value}))
            elif value.startswith(["launching+", "launching:"]):
                raise Exception("URN format only avialable online")
            value = json.loads(value)
        if isinstance(value, dict) and "path" in value and "urn" in value:
            return DataSet(json.dumps(value))
        else:
            raise Exception(f"DataSet Wrong type: {type(value)}")


class Model(str):
    def get_value(self) -> Dict[str, Any]:
        return json.loads(str(self))

    def get_path(self) -> str:
        return self.get_value()["path"]

    def get_full_path(self) -> str:
        path = self.get_path()
        return os.path.abspath(path)

    def get_urn(self) -> str:
        return self.get_value()["urn"]

    def get_subpath(self) -> str:
        return self.get_value()["subpath"]

    def get_name(self) -> str:
        return self.get_value().get("name")

    @classmethod
    def __modify_schema__(cls, field_schema: Dict[str, Any]) -> None:
        field_schema.update(format="model", scope="io")

    @classmethod
    def __get_validators__(cls) -> Any:
        yield cls.validate

    @classmethod
    def validate(cls, value: Any) -> "Model":
        if isinstance(value, Model):
            return value
        elif isinstance(value, str):
            if os.path.exists(value):
                return DataSet(json.dumps({"path": value, "urn": value, "subpath": ""}))
            elif value.startswith(["launching+", "launching:"]):
                raise Exception("URN format only avialable online")
            value = json.loads(value)
        if isinstance(value, dict) and "path" in value and "urn" in value:
            return Model(json.dumps(value))
        else:
            raise Exception(f"Model Wrong type: {type(value)}")


class InputMoleculeContent(str):
    def get_value(self) -> str:
        return str(self)

    @classmethod
    def __modify_schema__(cls, field_schema: Dict[str, Any]) -> None:
        field_schema.update(format="input_molecule_content", scope="io")

    @classmethod
    def __get_validators__(cls) -> Any:
        yield cls.validate

    @classmethod
    def validate(cls, value: Any) -> "InputMoleculeContent":
        if isinstance(value, InputMoleculeContent):
            return value
        elif isinstance(value, str):
            return InputMoleculeContent(value)
        else:
            raise Exception(f"InputMoleculeContent Wrong type: {type(value)}")



import base64
from typing import Any, Dict

__all__ = [
    "DflowArgoAPIServer",
    "DflowK8sAPIServer",
    "DflowAccessToken",
    "DflowStorageEndpoint",
    "DflowStorageRepository",
    "DflowLabels"
]


class DflowArgoAPIServer(str):
    def get_value(self):
        return str(self)

    @classmethod
    def __modify_schema__(cls, field_schema: Dict[str, Any]) -> None:
        field_schema.update(format="dflow_argo_apiserver", scope="executor")

    @classmethod
    def __get_validators__(cls) -> Any:
        yield cls.validate

    @classmethod
    def validate(cls, value: Any) -> "DflowArgoAPIServer":
        if isinstance(value, DflowArgoAPIServer):
            return value
        elif isinstance(value, str):
            return DflowArgoAPIServer(value)
        elif isinstance(value, (bytes, bytearray, memoryview)):
            return DflowArgoAPIServer(base64.b64encode(value).decode())
        else:
            raise Exception(f"DflowArgoAPIServer Wrong type: {type(value)}")


DflowHost = DflowArgoAPIServer


class DflowK8sAPIServer(str):
    def get_value(self):
        return str(self)

    @classmethod
    def __modify_schema__(cls, field_schema: Dict[str, Any]) -> None:
        field_schema.update(format="dflow_k8s_apiserver", scope="executor")

    @classmethod
    def __get_validators__(cls) -> Any:
        yield cls.validate

    @classmethod
    def validate(cls, value: Any) -> "DflowK8sAPIServer":
        if isinstance(value, DflowK8sAPIServer):
            return value
        elif isinstance(value, str):
            return DflowK8sAPIServer(value)
        elif isinstance(value, (bytes, bytearray, memoryview)):
            return DflowK8sAPIServer(base64.b64encode(value).decode())
        else:
            raise Exception(f"DflowK8sAPIServer Wrong type: {type(value)}")


class DflowAccessToken(str):
    def get_value(self):
        return str(self)

    @classmethod
    def __modify_schema__(cls, field_schema: Dict[str, Any]) -> None:
        field_schema.update(format="dflow_access_token", scope="executor")

    @classmethod
    def __get_validators__(cls) -> Any:
        yield cls.validate

    @classmethod
    def validate(cls, value: Any) -> "DflowAccessToken":
        if isinstance(value, DflowAccessToken):
            return value
        elif isinstance(value, str):
            return DflowAccessToken(value)
        elif isinstance(value, (bytes, bytearray, memoryview)):
            return DflowAccessToken(base64.b64encode(value).decode())
        else:
            raise Exception(f"DflowAccessToken Wrong type: {type(value)}")


class DflowStorageEndpoint(str):
    def get_value(self):
        return str(self)

    @classmethod
    def __modify_schema__(cls, field_schema: Dict[str, Any]) -> None:
        field_schema.update(format="dflow_storage_endpoint", scope="executor")

    @classmethod
    def __get_validators__(cls) -> Any:
        yield cls.validate

    @classmethod
    def validate(cls, value: Any) -> "DflowStorageEndpoint":
        if isinstance(value, DflowStorageEndpoint):
            return value
        elif isinstance(value, str):
            return DflowStorageEndpoint(value)
        elif isinstance(value, (bytes, bytearray, memoryview)):
            return DflowStorageEndpoint(base64.b64encode(value).decode())
        else:
            raise Exception(f"DflowStorageEndpoint Wrong type: {type(value)}")


class DflowStorageRepository(str):
    def get_value(self):
        return str(self)

    @classmethod
    def __modify_schema__(cls, field_schema: Dict[str, Any]) -> None:
        field_schema.update(format="dflow_storage_repository", scope="executor")

    @classmethod
    def __get_validators__(cls) -> Any:
        yield cls.validate

    @classmethod
    def validate(cls, value: Any) -> "DflowStorageRepository":
        if isinstance(value, DflowStorageRepository):
            return value
        elif isinstance(value, str):
            return DflowStorageRepository(value)
        elif isinstance(value, (bytes, bytearray, memoryview)):
            return DflowStorageRepository(base64.b64encode(value).decode())
        else:
            raise Exception(f"DflowStorageRepository Wrong type: {type(value)}")


class DflowLabels(dict):
    def get_value(self) -> Dict:
        if hasattr(self, "value"):
            return self.value
        return {}

    @classmethod
    def __modify_schema__(cls, field_schema: Dict[str, Any]) -> None:
        field_schema.update(format="dflow_labels", scope="executor")

    @classmethod
    def __get_validators__(cls) -> Any:  # type: ignore
        yield cls.validate

    @classmethod
    def validate(cls, value: Any) -> "DflowLabels":
        if isinstance(value, dict):
            f = DflowLabels(value)
            f.value = value
            return f
        else:
            raise Exception(f"DflowLabels Wrong type: {type(value)}")



import base64
from typing import Any, Dict, List, Tuple
import json

__all__ = ["SelectPocket"]


class SelectPocket(str):
    def get_value(self) -> Dict:
        return self.value

    def get_name(self) -> str:
        return self.value.get("name", "")

    def get_rank(self) -> int:
        return int(self.value.get("rank", -1))

    def get_score(self) -> float:
        return float(self.value.get("score", -1))

    def get_probability(self) -> float:
        return float(self.value.get("probability", -1))

    def get_center(self) -> Tuple[float]:
        return tuple(self.value.get("center")) if "center" in self.value else None

    def get_residues(self) -> List[str]:
        return self.value.get("residues") or list()

    def get_surface(self) -> List[str]:
        return self.value.get("surface") or list()

    @classmethod
    def __modify_schema__(cls, field_schema: Dict[str, Any]) -> None:
        field_schema.update(format="select_pocket", scope="io")

    @classmethod
    def __get_validators__(cls) -> Any:
        yield cls.validate

    @classmethod
    def validate(cls, value: Any) -> "SelectPocket":
        if isinstance(value, SelectPocket):
            return value
        elif isinstance(value, str):
            p = SelectPocket(value)
            p.value = json.loads(value)
            return p
        elif isinstance(value, dict):
            p = SelectPocket(json.dumps(value))
            p.value = value
            return p
        elif isinstance(value, (bytes, bytearray, memoryview)):
            value = base64.b64encode(value).decode()
            p = SelectPocket(value)
            p.value = json.loads(value)
            return p
        else:
            raise Exception(f"SelectPocket Wrong type: {type(value)}")
